require 'rspec-puppet/matchers/create_generic'
require 'rspec-puppet/matchers/include_class'
require 'rspec-puppet/matchers/compile'
require 'rspec-puppet/matchers/run'
require 'rspec-puppet/matchers/count_generic'
require 'rspec-puppet/matchers/dynamic_matchers'
